@@ -1,7 +1,7 @@
 ////////////////////////////////////////////////////////////////////////////
 // Porgram: FfCvt
 // Purpose: ffmpeg convert wrapper tool
-// Authors: Tong Sun (c) 2015-2019, All rights reserved
+// Authors: Tong Sun (c) 2015-2022, All rights reserved
 ////////////////////////////////////////////////////////////////////////////
 
 /*
@@ -41,15 +41,15 @@ const _encodedExt = "_.mkv"
 // Global variables definitions
 
 var (
-	version = "1.5.05"
-	date    = "2020-01-01"
+	version = "1.7.5"
+	date    = "2022-01-02"
 
-	sprintf           = fmt.Sprintf
 	encodedExt string = _encodedExt
 	totalOrg   int64  = 1
 	totalNew   int64  = 1
 	videos     []string
 	workDirs   []string
+	cutOps     string = ""
 )
 
 ////////////////////////////////////////////////////////////////////////////
@@ -64,6 +64,65 @@ func main() {
 		os.Exit(0)
 	}
 
+	if len(Opts.Seg) > 0 {
+		// sanity check
+		_, err := time.Parse("15:04:05", Opts.Seg)
+		if err != nil {
+			fmt.Fprintf(os.Stderr, "Seg format error: '%s'\n", Opts.Seg)
+			os.Exit(1)
+		}
+	}
+
+	if len(Opts.Cut) > 0 {
+		var b, vc strings.Builder
+		var ci int
+		var cv string
+		d0, _ := time.Parse("15:04:05", "00:00:00")
+		for ii, val := range Opts.Cut {
+			ci = ii
+			cv = val
+			cRange := regexp.MustCompile(`\s*(.*?)\s*-\s*(\S*)\s*$`).
+				FindStringSubmatch(val)
+			if len(cRange) != 3 {
+				fmt.Fprintf(os.Stderr, "pair - ")
+				goto range_error
+			}
+			//fmt.Println("Cut range:", cRange[1], cRange[2])
+			timeBgn, err := time.Parse("15:04:05", cRange[1])
+			if err != nil {
+				//fmt.Fprintf(os.Stderr, err.Error())
+				fmt.Fprintf(os.Stderr, "start - ")
+				goto range_error
+			}
+			endStr := ""
+			if len(cRange[2]) > 0 {
+				timeEnd, err := time.Parse("15:04:05", cRange[2])
+				if err != nil {
+					//fmt.Fprintf(os.Stderr, err.Error())
+					fmt.Fprintf(os.Stderr, "end - ")
+					goto range_error
+				}
+				endStr = fmt.Sprintf(":end=%d", int(timeEnd.Sub(d0).Seconds()))
+			}
+			secBgn := int(timeBgn.Sub(d0).Seconds())
+			fmt.Fprintf(&b, "[0:v]trim=start=%d%s,setpts=PTS-STARTPTS[v%d];"+
+				"[0:a]atrim=start=%d%s,asetpts=PTS-STARTPTS[a%d];",
+				secBgn, endStr, ii, secBgn, endStr, ii)
+			fmt.Fprintf(&vc, "[v%d][a%d]", ii, ii)
+		}
+
+		//fmt.Println("Cut(s):", len(Opts.Cut), Opts.Cut)
+		//fmt.Println(vc.String())
+		fmt.Fprintf(&b, "%sconcat=n=%d:v=1:a=1[vo][ao]", vc.String(), len(Opts.Cut))
+		//fmt.Println(b.String())
+		cutOps = b.String()
+		goto cut_ok
+	range_error:
+		fmt.Fprintf(os.Stderr, "Cut range %d format error for '%s'\n", ci, cv)
+		os.Exit(1)
+	}
+cut_ok:
+
 	// One mandatory arguments, either -d or -f
 	if len(Opts.Directory)+len(Opts.File) < 1 {
 		Usage()
@@ -214,7 +273,10 @@ func transcodeVideos(startTime time.Time) {
 
 func transcodeFile(inputName string) {
 	startTime := time.Now()
-	outputName := getOutputName(inputName)
+	outputName, outputGrpName := getOutputName(inputName), ""
+	if len(Opts.Seg) > 0 {
+		outputName, outputGrpName = getOutputNameSeg(inputName)
+	}
 	debug(outputName, 4)
 	os.MkdirAll(filepath.Dir(outputName), os.ModePerm)
 	var oldAEP, oldVEP, oldSEP string
@@ -228,35 +290,35 @@ func transcodeFile(inputName string) {
 			return
 		}
 		debug(fsinfo, 4)
-		// if there are more than one audio stream
+		/*
+
+		   Cases when `-map`s are necessary
+
+		   - more than one audio stream, and we pick eng stream only
+		   - more than one subtitle stream, and
+		     * output all subtitle streams (default, no -sel), or
+		     * pick specific subtitle stream(s) via -sel
+
+		*/
 		allAudioStreams := regexp.MustCompile(`Stream #0:.+: Audio: (.+)`).
 			FindAllStringSubmatch(fsinfo, -1)
-		if len(allAudioStreams) > 1 {
-			// then find the designated audio stream language
-			audioStreams := regexp.
-				MustCompile(`Stream #(.+)\(` + Opts.Lang + `\): Audio: (.+)`).
-				FindStringSubmatch(fsinfo)
-			if len(audioStreams) >= 1 {
-				// and use the 1st audio stream of the designated language
-				// via *temporarily* tweaking AEP/VEP/SEP setting
-				oldAEP, oldVEP, oldSEP = Opts.AEP, Opts.VEP, Opts.SEP
-				oldEPUsed = true
-				debug(audioStreams[1], 3)
-
-				Opts.AEP += " -map " + audioStreams[1]
-				dealSurroundSound(audioStreams[2])
-				// when `-map` is used (for audio), then all else need mapping as well
-				videoStreams := regexp.MustCompile(`Stream #(.+?)(\(.+\))*: Video: `).
-					FindStringSubmatch(fsinfo)
-				Opts.VEP += " -map " + videoStreams[1]
-				subtitleStreams := regexp.MustCompile(`Stream #(.+)\(.+\): Subtitle: `).
-					FindAllStringSubmatch(fsinfo, -1)
-				// keep all subtitle streams
-				for _, subtitleStream := range subtitleStreams {
-					Opts.SEP += " -map " + subtitleStream[1]
+		if len(allAudioStreams) > 1 ||
+			len(regexp.MustCompile(`Stream #0:.+: Subtitle: (.+)`).
+				FindAllStringSubmatch(fsinfo, -1)) > 1 {
+			// then use the designated audio stream language
+			// via *temporarily* using the AEP/VEP/SEP setting
+			oldAEP, oldVEP, oldSEP = Opts.AEP, Opts.VEP, Opts.SEP
+			oldEPUsed = true
+			Opts.VEP += " -map 0:v"
+			Opts.AEP += " -map 0:a:m:language:" + Opts.Lang
+			//log.Printf("%s: Opts.SEL - %#v", progname, Opts.SEL)
+			if len(Opts.SEL) == 0 {
+				Opts.SEP += " -map 0:s"
+			} else {
+				for _, val := range Opts.SEL {
+					Opts.SEP += " -map 0:s:m:language:" + val
 				}
 			}
-			// else: designated audio language not found, use `default` instead
 		} else {
 			debug(inputName+" has single audio stream", 2)
 			dealSurroundSound(allAudioStreams[0][1])
@@ -269,8 +331,37 @@ func transcodeFile(inputName string) {
 	if Opts.Force {
 		args = append(args, "-y")
 	}
+	if len(Opts.Seg) > 0 {
+		args = append(args, "-f")
+		args = append(args, "segment")
+		args = append(args, "-segment_time")
+		args = append(args, Opts.Seg)
+		args = append(args, "-reset_timestamps")
+		args = append(args, "1")
+	}
+	if len(Opts.Speed) > 0 {
+		args = append(args, "-filter_complex")
+		args = append(args, fmt.Sprintf("[0:v]setpts=PTS/%s[v];[0:a]atempo=%s[a]",
+			Opts.Speed, Opts.Speed))
+		args = append(args, "-map")
+		args = append(args, "[v]")
+		args = append(args, "-map")
+		args = append(args, "[a]")
+	}
+	if len(cutOps) != 0 {
+		args = append(args, "-filter_complex")
+		args = append(args, cutOps)
+		args = append(args, "-map")
+		args = append(args, "[vo]")
+		args = append(args, "-map")
+		args = append(args, "[ao]")
+	}
 	args = append(args, flag.Args()...)
-	args = append(args, outputName)
+	if len(Opts.Seg) > 0 {
+		args = append(args, outputGrpName)
+	} else {
+		args = append(args, outputName)
+	}
 	debug(Opts.FFMpeg+" "+strings.Join(args, " "), 1)
 
 	if Opts.NoExec {
@@ -417,7 +508,7 @@ func encodeParametersV(args []string) []string {
 			// -b:v 0 -crf 37
 			args = append(args, "-b:v", "0", "-crf", Opts.CRF)
 		}
-		if Opts.VES[:6] == "libx26" {
+		if len(Opts.VES) > 6 && Opts.VES[:6] == "libx26" {
 			args = append(args, "-"+Opts.VES[3:]+"-params", "crf="+Opts.CRF)
 		}
 	}
@@ -471,6 +562,16 @@ func getOutputName(input string) string {
 	return r
 }
 
+// getOutputNameSeg will do getOutputName() but tailored toward video segmenting.
+// The first return will be the first video segment file name, 00_.mkv, while
+// the second return will be the segment group file name, %02d_.mkv
+func getOutputNameSeg(input string) (string, string) {
+	r := getOutputName(input)
+	fileFirst := strings.Replace(r, encodedExt, "00"+encodedExt, 1)
+	fileGroup := strings.Replace(r, encodedExt, "%02d"+encodedExt, 1)
+	return fileFirst, fileGroup
+}
+
 //==========================================================================
 // Support functions
 
